% lab11_ex_differentiation_fm1.m
% LOW-PASS AND DIFFERENTIATION FILTERS - DEMO: APPLICATION FOR FM DE-MODULATION
%
% SUMMARY
% The goal of this demonstration program is to show interesting practical application
% of two types of digital filters: differentiation and low-pass.
% The reader is encouraged to replace differentiation and low-pass filters designed by
% Matlab functions by his own manual designs exploiting known analytic solutions used
% in window method.
%
% In digital receivers of FM-modulated signals the following method can be used.
% Cosine carrier modulated in frequency by x(t) is given by:
%    c(t) =cos(2*pi*fc*t + fi(t)),  fi(t)=2*pi*kf*cumsum(x(t))*dt
% where kf denotes modulation depth (width).
% To recover x(t) having c(t) it is possible to differentiate the signal c(t):
%   dc(t)/dt =-(2*pi*fc + 2*pi*kf*x(t)) * sin(2*pi*fc*t + fi(t)) = -e(t) * sin(.),
% and detect the slow-changing envelope e(t) of fast-changing sin().
% For this purpose we can:
% 1) square the derivative:
%    e(t)^2 * sin(a)^2 = e(t)^2 * 0.5*(1-cos(2a)) = 0.5*e(t)^2 - 0.5*e(t)^2*cos(2a),
% 2) low-pass filter the result and remove this way the second high-freq component:
%    0.5*e(t)^2
% 3) multiply by 2 and take the square root:
%    e(t) = 2*pi*(fc + kf*x(t))
% 4) divide by 2*pi, subtract fc and divide by kf:
%    x(t)
% Two filters are needed: band-pass diff around fc (+/-DF) and low-pass 4 kHz.

% 8 kHz speech is up-sampled and madulate a cosine with fc in frequency

  clear all; close all;

  ifigs = 1;        % 0/1 Yes/No

  Nx = 8000;        % number of input 8 kHz speech samples to be processed
  M = 200; N=2*M+1; % N=2M+1: length of used FIR filters (to be designed)
  K  = 5;           % speech up-sampling ratio
  fc = 8000;        % carrier frequency (to be changed by speech signal)
  fs = K*fc;                     % sampling frequency after up-sampling 
  Nfigs = 2^14; nn = 1:Nfigs;    % for figures

% GENERATION OF FM-MODULATED CARRIER  
  
% Read speech 8kHz signal to be FM-broadcasted  
  [x, fx ] = audioread('speech.wav',[1,Nx]); x=x.'; % plot(x);
      if(ifigs==1) figure;
      subplot(211); plot(x); grid; title('x(n) input');
      subplot(212); specgram(x,512,fs,256,192); title('STFT(1)'); pause % Matlab (old), Octave
      end      
% Up-sampling signal to frequency fs
  x = interp(x,K); Nx = length(x);
      if(ifigs==1) figure;
      subplot(211); plot(x(1:Nfigs)); title('x(n) interpolated');
      subplot(212); specgram(x(nn),512,fs,256,192); title('STFT(2)'); pause % Matlab (old), Octave
      end    
% Final frequency modulation of the carrier by x(n)
  fmax = 4000;            % maximum signal frequency of speech (Hz)
  DF = 5000;              % 2*DF = required bandwidth of FM modulated signal (Hz)
  kf = (DF/fmax-1)*fmax;  % modulation index from Carson's rule

  x = cos( 2*pi*( fc/fs*(0:Nx-1) + kf*cumsum(x)/fs ) ); x=x.'; clear n;
      if(ifigs==1) figure;
      subplot(211); plot(x(1:Nfigs)); title('x(n) after FM');
      subplot(212); specgram(x(nn),512,fs,256,192); title('STFT(3)'); pause % Matlab (old), Octave
      end
  
 % FM DEMULATION USING DIFFERENTIATION FILTER

  y = x; clear x;

% Designing impulse response of the FIR differentiator using window method
 n=-M:M; hD=cos(pi*n)./n; hD(M+1)=0; w = kaiser(N,10)'; hD = hD .* w;
      if(ifigs==1) figure;
      f=0:fs/1000:fs/2;
      H=freqz(hD,1,f,fs); H=H.*exp(j*2*pi*f/fs*M);
      subplot(311); stem(hD); title('hD(n)');
      subplot(312); plot(f,20*log10(abs(H))); title('DIFF abs(H(f))');
      subplot(313); plot(f,unwrap(angle(H))); title('DIFF angle(H(f))'); pause
      end   

% Differentiation
  y = filter(hD, 1, y);  y = y(N:end);
      if(ifigs==1) figure;
      subplot(211); plot(y(1:Nfigs)); title('y(n) after differentiator');
      subplot(212); specgram(y(nn),512,fs,256,192); title('STFT(4)'); pause % Matlab (old), Octave
      end
  
% Power of 2  
  y = y.^2;
      if(ifigs==1) figure;
      subplot(211); plot(y(1:Nfigs)); title('y(n) after abs()');
      subplot(212); specgram(y(nn),512,fs,256,192); title('STFT(5)'); pause % Matlab (old), Octave
      end
  
% Designing impulse response of the low-pass FIR filter
  n=-M:M; hLP=sin(2*pi*4000/fs*n)./(pi*n); hLP(M+1)=2*(4000)/fs; hLP = hLP .* w;
      if(ifigs==1) figure;
      H=freqz(hLP,1,f,fs);
      subplot(311); stem(hLP); title('hLP(n)');
      subplot(312); plot(f,20*log10(abs(H))); title('LowPass abs(H(f))');
      subplot(313); plot(f,unwrap(angle(H))); title('LowPass angle(H(f))'); pause
      end
  
% Low-pass filtration  
  y = filter(hLP, 1, y ); y = y(N:end);
      if(ifigs==1) figure;
      subplot(211); plot(y(1:Nfigs)); title('y(n) after Low-Pass filter');
      subplot(212); specgram(y(nn),512,fs,256,192); title('STFT(6)'); pause % Matlab (old), Octave
      end
  
% Decimation and square root  
  y = real( sqrt( 2 * y(1:K:end) ) );
  y = (y - 2*pi*fc/fs)/(2*pi*kf/fs);
      if(ifigs==1) figure;
      subplot(211); plot(y); grid; title('y(n) after decimation and SQRT()');
      subplot(212); specgram(y,512,fs,256,192); title('STFT(7)'); pause % Matlab (old), Octave
      end
  
% Let the music play  
  soundsc(y,fx); pause